////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2016 Clemson University.
//
// This file was originally written by Bradley S. Meyer.
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this software; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
// USA
//
//////////////////////////////////////////////////////////////////////////////*/

////////////////////////////////////////////////////////////////////////////////
//!
//! \file
//! \brief Example code for computing user-defined thermo quantity.
//!
////////////////////////////////////////////////////////////////////////////////

#include "my_user/my_thermo.h"

namespace my_user
{

//############################################################################
// my_baryon_entropy().
//############################################################################

double
my_baryon_entropy(
  nnt::Zone& zone
)
{

  double d_result, d_ye;
  boost::any nse_corr_data;

  d_result = user::compute_baryon_entropy_per_nucleon( zone );

  d_ye = Libnucnet__Zone__computeZMoment( zone.getNucnetZone(), 1 );

  nse_corr_data = user::get_nse_correction_data( );

  nnt::species_list_t species_list =
    nnt::make_species_list(
      Libnucnet__Net__getNuc(
        Libnucnet__Zone__getNet( zone.getNucnetZone() )
      )
    );

  BOOST_FOREACH( nnt::Species species, species_list )
  {
    d_result +=
      user::species_coulomb_entropy(
        species.getNucnetSpecies(),
        zone.getProperty<double>( nnt::s_T9 ),
        zone.getProperty<double>( nnt::s_RHO ),
        d_ye,
        &nse_corr_data
      ) *
      Libnucnet__Zone__getSpeciesAbundance(
        zone.getNucnetZone(),
        species.getNucnetSpecies()
      );
  }

  return d_result;

}

//############################################################################
// my_total_entropy().
//############################################################################

double
my_total_entropy(
  nnt::Zone& zone
)
{

  return
    my_baryon_entropy( zone ) +
    user::compute_photon_entropy_per_nucleon( zone ) +
    user::compute_electron_entropy_per_nucleon( zone );

}

} // my_user
